using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace Azbest
{
    public class Board
    {
        public RotatedTile [,] Tiles = null;

        public int BoardSize;

        public readonly object BoardLock = new object();

        public event MethodInvoker RefreshViewControl = null;

        public Board()
        {
            InitializeBoard(6);
        }
        
        private void InitializeBoard(int size)
        {
            lock (BoardLock)
            {
                BoardSize = size;
                Tiles = new RotatedTile[BoardSize, BoardSize];
                for (int iY = 0; iY < BoardSize; ++iY)
                {
                    for (int iX = 0; iX < BoardSize; ++iX)
                    {
                        Tiles[iX, iY] = null;
                    }
                }
            }
        }

        public void Draw(Graphics graph, int totalSize)
        {
            lock (BoardLock)
            {
                try
                {
                    int tileSize = totalSize / BoardSize;
                    graph.Clear(Color.Black);

                    for (int iY = 0; iY < BoardSize; ++iY)
                    {
                        for (int iX = 0; iX < BoardSize; ++iX)
                        {
                            if (Tiles[iX, iY] != null)
                            {
                                TileDrawer.DrawTile(Tiles[iX, iY], graph, iX * tileSize, iY * tileSize, tileSize);
                            }
                            else
                            {
                                TileDrawer.DrawEmptyTile(graph, iX * tileSize, iY * tileSize, tileSize);
                            }
                        }
                    }
                }
                catch { }
            }
        }

        public void Generate(int size, int maxInteriorColors, int maxBorderColors, bool rotateAndShufflePieces, bool useRandom, bool uniqueOnly)
        {
            lock (BoardLock)
            {
                Random rand = new Random();
                if (!useRandom)
                {
                    rand = new Random(0);
                }
                GameManager.Instance.Colors = maxInteriorColors + maxBorderColors + 1;
                InitializeBoard(size);
                List<Tile> createdTiles = new List<Tile>();

                for (int iY = 0; iY < BoardSize; ++iY)
                {
                    for (int iX = 0; iX < BoardSize; ++iX)
                    {
                        bool remakeTile = true;
                        int retries = 0;

                        int leftColor = 0;
                        int upColor = 0;
                        int rightColor = 0;
                        int downColor = 0;

                        Tile createdTile = null;

                        while (remakeTile)
                        {
                            leftColor = 0;
                            if (iX > 0) leftColor = Tiles[iX - 1, iY].RightColor;
                            rightColor = rand.Next(maxInteriorColors) + 1;
                            if ((iY == 0) || (iY == BoardSize - 1)) rightColor = rand.Next(maxBorderColors) + 1 + maxInteriorColors;
                            if (iX == BoardSize - 1) rightColor = 0;

                            upColor = 0;
                            if (iY > 0) upColor = Tiles[iX, iY - 1].BottomColor;
                            downColor = rand.Next(maxInteriorColors) + 1;
                            if ((iX == 0) || (iX == BoardSize - 1)) downColor = rand.Next(maxBorderColors) + 1 + maxInteriorColors;
                            if (iY == BoardSize - 1) downColor = 0;

                            createdTile = new Tile(leftColor, upColor, rightColor, downColor);

                            if (uniqueOnly)
                            {
                                bool isUnique = true;
                                DrawableSquareComparer comparer = new DrawableSquareComparer();
                                for (int i = 0; i < createdTiles.Count; ++i)
                                {
                                    if (comparer.Equals(createdTiles[i], createdTile))
                                    {
                                        isUnique = false;
                                        break;
                                    }
                                }
                                if (isUnique)
                                {
                                    remakeTile = false;
                                    createdTiles.Add(createdTile);
                                }
                            }
                            else
                            {
                                remakeTile = false;
                            }
                            ++retries;
                            if (retries > 500000)
                            {
                                MessageBox.Show("Nie udalo sie stworzyc unikatowej planszy!");
                                uniqueOnly = false;
                                remakeTile = false;
                            }
                        }

                        Tiles[iX, iY] = new RotatedTile(createdTile, 0);
                    }
                }

                if (rotateAndShufflePieces)
                {
                    for (int iY = 0; iY < BoardSize; ++iY)
                    {
                        for (int iX = 0; iX < BoardSize; ++iX)
                        {
                            Tiles[iX, iY].BaseTile.RotateForGood((uint)rand.Next(4));
                        }
                    }
                    for (int iY = 0; iY < BoardSize; ++iY)
                    {
                        for (int iX = 0; iX < BoardSize; ++iX)
                        {
                            RotatedTile tmp = Tiles[iX, iY];
                            int randX = rand.Next(BoardSize);
                            int randY = rand.Next(BoardSize);
                            Tiles[iX, iY] = Tiles[randX, randY];
                            Tiles[randX, randY] = tmp;
                        }
                    }
                }
            }
        }

        public void DeleteAllTiles()
        {
            lock (BoardLock)
            {
                for (int iY = 0; iY < BoardSize; ++iY)
                {
                    for (int iX = 0; iX < BoardSize; ++iX)
                    {
                        Tiles[iX, iY] = null;
                    }
                }
            }
        }

        public void MoveAllTilesToPool(FreeTiles freeTiles)
        {
            lock (BoardLock)
            {
                for (int iY = 0; iY < BoardSize; ++iY)
                {
                    for (int iX = 0; iX < BoardSize; ++iX)
                    {
                        freeTiles.Tiles.Add(Tiles[iX, iY]);
                    }
                }
                DeleteAllTiles();
            }
        }

        public Point GetIndexOfTile(RotatedTile tile)
        {
            for (int iY = 0; iY < BoardSize; ++iY)
            {
                for (int iX = 0; iX < BoardSize; ++iX)
                {
                    if (Tiles[iX, iY] == tile) return new Point(iX, iY);
                }
            }
            return new Point(-1, -1);
        }

        public RotatedTile GetTileAt(Point position, int controlWidth)
        {
            lock (BoardLock)
            {
                int tileSize = controlWidth / GameManager.Instance.CurrentBoard.BoardSize;

                int indexX = position.X / tileSize;
                int indexY = position.Y / tileSize;

                if (indexX >= BoardSize)
                {
                    indexX = BoardSize - 1;
                }

                if (indexY >= BoardSize)
                {
                    indexY = BoardSize - 1;
                }

                return Tiles[indexX, indexY];
            }
        }

        public Point GetTileIndexAtControlPoint(Point position, int controlWidth)
        {
            lock (BoardLock)
            {
                int tileSize = controlWidth / GameManager.Instance.CurrentBoard.BoardSize;

                int indexX = position.X / tileSize;
                int indexY = position.Y / tileSize;

                if (indexX >= BoardSize)
                {
                    indexX = BoardSize - 1;
                }

                if (indexY >= BoardSize)
                {
                    indexY = BoardSize - 1;
                }
                return new Point(indexX, indexY);
            }
        }

        public bool IsCompleted()
        {
            lock (BoardLock)
            {
                for (int iY = 0; iY < BoardSize; ++iY)
                {
                    for (int iX = 0; iX < BoardSize; ++iX)
                    {
                        if (Tiles[iX, iY] == null) return false;
                        if (iX > 0)
                        {
                            if (Tiles[iX, iY].LeftColor != Tiles[iX - 1, iY].RightColor) return false;
                        }
                        else
                        {
                            if (Tiles[iX, iY].LeftColor != 0) return false;
                        }
                        if (iX < BoardSize - 1)
                        {
                            if (Tiles[iX, iY].RightColor != Tiles[iX + 1, iY].LeftColor) return false;
                        }
                        else
                        {
                            if (Tiles[iX, iY].RightColor != 0) return false;
                        }

                        if (iY > 0)
                        {
                            if (Tiles[iX, iY].TopColor != Tiles[iX, iY - 1].BottomColor) return false;
                        }
                        else
                        {
                            if (Tiles[iX, iY].TopColor != 0) return false;
                        }
                        if (iY < BoardSize - 1)
                        {
                            if (Tiles[iX, iY].BottomColor != Tiles[iX, iY + 1].TopColor) return false;
                        }
                        else
                        {
                            if (Tiles[iX, iY].BottomColor != 0) return false;
                        }

                    }
                }
                return true;
            }
        }

        internal void RefreshView()
        {
            if (RefreshViewControl != null) RefreshViewControl();
        }
    }
}
